%
%
%  Filament analysis and Bouton and Spot finder
%
%  Copyright Bitplane BPI 2015
%  Matthew Gastinger, Ph.D
%  Application Support Scientist
%
%
%  Installation:
%
%  - Copy this file into the XTensions folder in the Imaris installation directory.
%  - You will find this function in the Image Processing menu
%
%    <CustomTools>
%      <Menu>
%       <Submenu name="Filaments Functions">
%        <Item name="Filament Analysis" icon="Matlab">
%          <Command>MatlabXT::XT_MJG_FilamentAnalysis(%i)</Command>
%        </Item>
%       </Submenu>
%      </Menu>
%      <SurpassTab>
%        <SurpassComponent name="bpFilaments">
%          <Item name="Filament Analysis" icon="Matlab">
%            <Command>MatlabXT::XT_MJG_FilamentAnalysis(%i)</Command>
%          </Item>
%        </SurpassComponent>
%      </SurpassTab>
%    </CustomTools>
% 
%
%  Description:
%       This XTension will do several things:
%       1)Place spots at each point along the filament, allowing for
%           visualizing and measuring the instensity along the dendrites
%       2)Find synaptic bouton(varicosities) along dendrite segments, place
%           a spot at that point.
%       3)Place spots at each of hte following places: 
%           beginning point,
%           dendrite branch, 
%           dendrite terminal point, 
%           Spine Attachment point
%           Spine terminal point
%       4)Generate a duplicate of the filament object with NEW statistics
%           Dendrite mean intensity (for each channel)
%           Spine intensity (not just the spine head, the whole spine)
%           Bouton(varicosity) number per dendrite segment
%           Bouton density
%           Spot Detection within a certain distance of filament
%           Spot Density
function XT_MJG_FilamentAnalysis(aImarisApplicationID)

% connect to Imaris interface
if ~isa(aImarisApplicationID, 'Imaris.IApplicationPrxHelper')
  javaaddpath ImarisLib.jar
  vImarisLib = ImarisLib;
  if ischar(aImarisApplicationID)
    aImarisApplicationID = round(str2double(aImarisApplicationID));
  end
  vImarisApplication = vImarisLib.GetApplication(aImarisApplicationID);
else
  vImarisApplication = aImarisApplicationID;
end


%Launch the GUI
%Collect of the data for the XTension
%Set some variables to global
global f qFindFilamentsPoints qDoBoutonDetection qDoSpotsCloseFilaments
global checkbox1 editbox1 checkbox2 editbox2 checkbox3 checkbox4
global vImarisApplicationGlobal
vImarisApplicationGlobal = vImarisApplication;

qDoBoutonDetection='NO'
qDoSpotsCloseFilaments='NO'
qFindFilamentsPoints='NO'

% Create figure
f = figure('units','pixels','position',[500,500,300,250],...
             'toolbar','none','menu','none',...
             'numbertitle','off','name','Filament Analysis');
% DetectBouton dialog
checkbox1 = uicontrol('Parent', f,'style','checkbox',...
    'position', [40,170,200,25],...
    'string', 'Detect Boutons(varicosities)');
%insert edit for checkbox1
editbox1 = uicontrol('Parent',f,'style','edit',...
    'position', [70,150,35,25],...
    'string','0.3');
%Text for check box1
text1 = uicontrol('Parent', f,'style','text',...
    'position',[110,145,50,25],...
    'string','sensitivity');

%Find Spots close to Filaments dialog line
checkbox2 = uicontrol('Parent', f,'style','checkbox',...
    'position', [40,120,200,25],...
    'string', 'Find Spots Close to Filaments');
%insert edit for checkbox2
editbox2 = uicontrol('Parent', f,'style','edit',...
    'position', [70,100,35,25],...
    'string', '1.0');
%Text for checkbox2
text2 = uicontrol('Parent', f,'style','text',...
    'position',[110,95,150,25],...
    'string','Spot Distance to Filaments (um)');

%Checkbox to run Filament Point detection
checkbox3 = uicontrol('Parent', f,'style','checkbox',...
    'position', [40,70,200,25],...
    'string', 'Filaments Points Detection');
%Text for checkbox3
text3 = uicontrol('Parent', f,'style','text',...
    'position',[73,45,200,30],...
    'string','Spots placed at: branch points, terminal points, spine attachments, spine terminals',...
    'fontangle','italic');
%Option to merge all filamentSPots into a single Spots object
%checkbox
checkbox4 = uicontrol('Parent', f,'style','checkbox',...
    'position', [40,205,200,25],...
    'string', 'Merge All filament Spots objects');




% Create ANAlYZE and Cancel pushbuttons  
analyzebutton = uicontrol('Parent', f,'style','pushbutton','units','pixels',...
                'position',[50,10,100,30],'string','ANALYZE',...
                'callback',@Analyze,...
                'fontweight','bold','fontSize', 14);
cancelbutton = uicontrol('Parent', f,'style','pushbutton','units','pixels',...
                'position',[170,15,70,20],'string','CANCEL',...
                'callback',@Cancel);
            
%End the Script after clicking CANCEL button
function Cancel(varargin)
global f 
delete (f);
return
                
% Analyze function - runs the XT script
function Analyze(varargin)
% GET data from the dialog menus
global vImarisApplicationGlobal qMergeSpots
global f qFindFilamentsPoints qDoBoutonDetection qDoSpotsCloseFilaments vDistanceLimit vPeakDetThreshold
global checkbox1 editbox1 checkbox2 editbox2 checkbox3 checkbox4
vImarisApplication = vImarisApplicationGlobal;

% connect to Imaris interface
if isa(vImarisApplication, 'Imaris.IApplicationPrxHelper')

%checkbox1
if get(checkbox1,'Value')==1;
    qDoBoutonDetection='YES'
else
    qDoBoutonDetection='NO'
end
vPeakDetThreshold=str2double(get(editbox1,'String'));
%checkbox2
if get(checkbox2,'Value')==1;
    qDoSpotsCloseFilaments='YES'
else
    qDoSpotsCloseFilaments='NO'
end
vDistanceLimit=str2double(get(editbox2,'String'));
%checkbox3
if get(checkbox3,'Value')==1;
    qFindFilamentsPoints='YES'
else
    qFindFilamentsPoints='NO'
end
%checkbox4
if get(checkbox4,'Value')==1;
    qMergeSpots='YES'
else
    qMergeSpots='NO'
end


clearvars checkbox1 editbox1 checkbox2 editbox2 checkbox3 checkbox4

% get the filament
vFactory = vImarisApplication.GetFactory;
vFilaments = vFactory.ToFilaments(vImarisApplication.GetSurpassSelection);
delete(f);
%%
%Create a new folder object for new Spots
result3 = vFactory.CreateDataContainer;
result3.SetName('New Filament Analysis');
result2 = vFactory.CreateDataContainer;
result2.SetName('All segments');
result1 = vFactory.CreateDataContainer;
result1.SetName('Filament displayed as Spots');
vNewSpots = vImarisApplication.GetFactory.CreateSpots;
vNewSpots2 = vImarisApplication.GetFactory.CreateSpots;
vNewSpots3 = vImarisApplication.GetFactory.CreateSpots;
vNewSpotsBouton = vImarisApplication.GetFactory.CreateSpots;
% search the filament if not previously selected
vSurpassScene = vImarisApplication.GetSurpassScene;
if ~vFactory.IsFilaments(vFilaments)        
    for vChildIndex = 1:vSurpassScene.GetNumberOfChildren
        vDataItem = vSurpassScene.GetChild(vChildIndex - 1);
        if vFactory.IsFilaments(vDataItem)
            vFilaments.SetVisible(0);
            vFilaments = vFactory.ToFilaments(vDataItem);
            break;
        end
    end
    % did we find the filament?
    if isequal(vFilaments, [])
        msgbox('Please create some filament!');
        return;
    end
end

vSpotsCount=0;
if isequal(qDoSpotsCloseFilaments,'YES')
    vSpots = vImarisApplication.GetFactory.ToSpots(vImarisApplication.GetSurpassSelection);
%Get the Spots Object of interest
    for vChildIndex = 1:vSurpassScene.GetNumberOfChildren
        vDataItem = vSurpassScene.GetChild(vChildIndex - 1);
        if isequal(vSpots, [])
            if vImarisApplication.GetFactory.IsSpots(vDataItem)
                vSpots = vImarisApplication.GetFactory.ToSpots(vDataItem);
            end 
        end
        if isequal(vFilaments, [])
            if vImarisApplication.GetFactory.IsFilaments(vDataItem)
                vFilaments = vImarisApplication.GetFactory.ToFilaments(vDataItem);
            end
        end
    end
%SelectSpotsObject
    vNumberOfSpots = 0;
    vSpotsList{vSurpassScene.GetNumberOfChildren} = [];
    vNamesList{vSurpassScene.GetNumberOfChildren} = [];
    for vChildIndex = 1:vSurpassScene.GetNumberOfChildren
        vDataItem = vSurpassScene.GetChild(vChildIndex - 1);
        if vImarisApplication.GetFactory.IsSpots(vDataItem)
            vNumberOfSpots = vNumberOfSpots+1;
            vSpotsList{vNumberOfSpots} = vImarisApplication.GetFactory.ToSpots(vDataItem);
            vNamesList{vNumberOfSpots} = char(vDataItem.GetName);
        end
    end

    vNamesList = vNamesList(1:vNumberOfSpots);
    if vNumberOfSpots > 1
        [vPair, vOk] = listdlg('ListString',vNamesList,'SelectionMode','single',...
            'ListSize',[250 150],'Name','Spots','InitialValue',1,...
            'PromptString',{'Please select spots object:'});
        if vOk<1, return, end
        vSpots = vSpotsList{vPair(1)};
    else
        vSpots = vSpotsList{1};
    end
    if vNumberOfSpots==0
        msgbox('Please create at a spots object to use this feature!');
        qDoSpotsCloseFilaments='NO';
        vSpotsCount=0;
    else
       % get the spot coordinates
        vSpotsPosXYZ = vSpots.GetPositionsXYZ;
        vSpotsRadius = vSpots.GetRadiiXYZ;
        vSpotsPosT = vSpots.GetIndicesT;
        vSpotsCount = size(vSpotsPosXYZ, 1); 
    end

end

%%
% create a structure to hold the data of one filament
% each element of the structure is a list of XYZR
vData = CollectFilamentsData(zeros(0, 3), [], zeros(0, 2), [], 0); 
vFieldNames = fieldnames(vData); 
vFieldsCount = numel(vFieldNames);

% create an array of structures (one element per time point)
vCount = vFilaments.GetNumberOfFilaments;
vTimes = 1:vCount;
for vIndex = 1:vCount
  vTimes(vIndex) = vFilaments.GetTimeIndex(vIndex-1)+1;
end
vSizeT = max(vTimes);
vData = vData(ones(vSizeT, 1));
%%
%
vNumberOfChannels=vImarisApplication.GetDataSet.GetSizeC;
vRGBA = vFilaments.GetColorRGBA
vNewFilament=vImarisApplication.GetFactory.CreateFilaments;
TotalBoutons=[];
TotalSegmentIndex=[];
TotalSpotsperSegment=[];
vBoutonPositionAll=[];
vBoutonTimeAll=[];
vBoutonRadiusAll=[];
CompleteTempMean=[];
CompleteTempCenter=[];
CompletewDendriteSegment=[];
CompleteCount=[];
CompletewSpineSegment=[];
CompletespTempMean=[];
CompletespTempCenter=[];
CompletespCount=[];
CompletevNumberofSpotsDendrite=[];
CompleteMeanFinal=[];
CompletespMeanFinal=[];
CompleteCenterFinal=[];
CompletespCenterFinal=[];
vIsCloseFinal=[];
TotalSpotsDendrite=[];
TotalSpotsDendriteRadius=[];
TotalSpotsDendriteTime=[];
TotalSpotsSpine=[];
TotalSpotsSpineRadius=[];
TotalSpotsSpineTime=[];
bar1=0;
bar2=0;
vFilamentPointsXYZFinal=[];
vFilamentPointsRadiusFinal=[];

vProgressDisplayDendrites= waitbar(0, 'Dendrite analysis','Position',[600 500 370 60]);

for FilamentIndex=0:vCount-1

    vNewSpots3.SetColorRGBA((rand(1, 1)) * 256 * 256 * 256 );
    vNewSpots3.SetName([char(vFilaments.GetName), ' DendriteSpots ' num2str(FilamentIndex)]);


    vFilamentsIndexT = vFilaments.GetTimeIndex(FilamentIndex);
    vFilamentsXYZ = vFilaments.GetPositionsXYZ(FilamentIndex);
    vFilamentsRadius = vFilaments.GetRadii(FilamentIndex);
    vFilamentsEdgesSegmentId = vFilaments.GetEdgesSegmentId(FilamentIndex);
    vSegmentIds=unique(vFilamentsEdgesSegmentId)%Idenitfy unique filament segmentIDs
    vBoutonPositionXYZFinal=[];
    vBoutonRadiusFinal=[];
    vBoutonTimeFinal=[];
    vBoutonNumberFinal=[];
    qIsAFilament='YES';
    %Test if the time point has empty filament matrix or filament start
    %point and nothing more
    IsFilamentTest = numel(vFilamentsRadius);
    if IsFilamentTest==1;
        qIsAFilament='NO';
    end
    if isempty(vFilamentsXYZ), 
        qIsAFilament='NO';
    end
        
    vNumberOfDendriteBranches = numel(vSegmentIds)%total number dendrite segments
    vNumberOfFilamentPoints= numel(vFilamentsRadius);%including starting point
    vTimeIndex(1:vNumberOfFilamentPoints)=vFilamentsIndexT;%for filament spot creation
    vFilamentsEdgesCopy=vFilaments.GetEdges(FilamentIndex);
    vFilamentsEdges = vFilaments.GetEdges(FilamentIndex) + 1;
    vTypes = vFilaments.GetTypes(FilamentIndex);
    vBeginningVertex = vFilaments.GetBeginningVertexIndex(FilamentIndex) + 1;

%%
    if isempty(vFilamentsXYZ)
        vFilamentsXYZ = zeros(0, 3);
    end
    if isempty(vFilamentsEdges)
        vFilamentsEdges = zeros(0, 2);
    end
    
    vThisData = CollectFilamentsData(vFilamentsXYZ, vFilamentsRadius, ...
        vFilamentsEdges, vTypes, vBeginningVertex);

    vTime = vTimes(FilamentIndex + 1);
    for vField = 1:vFieldsCount
        vFieldName = vFieldNames{vField};
        if isempty(vData(vTime).(vFieldName))
            vData(vTime).(vFieldName) = vThisData.(vFieldName);
        elseif ~isempty(vThisData.(vFieldName))
            vData(vTime).(vFieldName) = [vData(vTime).(vFieldName); vThisData.(vFieldName)];
        end
    end
    
    %%    
%Generate Spots-shaped filament to measure intensity and depth code
    vNewSpots = vImarisApplication.GetFactory.CreateSpots;
    vNewSpotsDendrite = vImarisApplication.GetFactory.CreateSpots;
    vNewSpotsSpine = vImarisApplication.GetFactory.CreateSpots;
    
%Adjusting the Filament Points that fall outside the edge of the volume for
%Spot placement and calculation of Intensities
    if FilamentIndex==0

    end
    
    
    vFilamentsT=vTypes==0;
    vDendriteIndex = find(vFilamentsT');
    vSpineIndex = find(vTypes');
    vDendriteXYZ=vFilamentsXYZ(vDendriteIndex,:);
    vSpineXYZ=vFilamentsXYZ(vSpineIndex,:);
    time=vTimeIndex';
    
    %Test whether to merge spot or not
    if isequal (qMergeSpots, 'YES')
        TotalSpotsDendrite=[TotalSpotsDendrite;vFilamentsXYZ(vDendriteIndex,:)];
        TotalSpotsDendriteTime=[TotalSpotsDendriteTime;time(vDendriteIndex,:)];
        TotalSpotsDendriteRadius=[TotalSpotsDendriteRadius;vFilamentsRadius(vDendriteIndex,:)];
        if vCount==FilamentIndex+1
            vNewSpotsDendrite.Set(TotalSpotsDendrite,TotalSpotsDendriteTime',TotalSpotsDendriteRadius);
            vNewSpotsDendrite.SetName([char(vFilaments.GetName), ' filaments all']);
            vNewSpotsDendrite.SetColorRGBA((rand(1, 1)) * 256 * 256 * 256 );
            result1.AddChild(vNewSpotsDendrite, -1);
        end
        if ~isempty (vSpineXYZ)
            TotalSpotsSpine=[TotalSpotsSpine;vFilamentsXYZ(vSpineIndex,:)];
            TotalSpotsSpineRadius=[TotalSpotsSpineRadius;vFilamentsRadius(vSpineIndex,:)];
            TotalSpotsSpineTime=[TotalSpotsSpineTime;time(vSpineIndex,:)];
            if vCount==FilamentIndex+1    
                vNewSpotsSpine.Set(TotalSpotsSpine,TotalSpotsSpineTime',TotalSpotsSpineRadius);
                vNewSpotsSpine.SetName([char(vFilaments.GetName), ' spines all']);
                vNewSpotsSpine.SetColorRGBA((rand(1, 1)) * 256 * 256 * 256 );
                result1.AddChild(vNewSpotsSpine, -1);
            end
        end
    else    
    %create dendrite spots object
    if isequal (qIsAFilament,'YES')
        vNewSpotsDendrite.Set(vFilamentsXYZ(vDendriteIndex,:), time(vDendriteIndex,:)', vFilamentsRadius(vDendriteIndex,:));
        vNewSpotsDendrite.SetColorRGBA((rand(1, 1)) * 256 * 256 * 256 );
        vNewSpotsDendrite.SetName([char(vFilaments.GetName), ' dendrites ' num2str(FilamentIndex)]);
        result1.AddChild(vNewSpotsDendrite, -1);
    end
    %create spine spots object
    if ~isempty (vSpineXYZ) & isequal (qIsAFilament,'YES')
        vNewSpotsSpine.Set(vFilamentsXYZ(vSpineIndex,:), time(vSpineIndex,:)', vFilamentsRadius(vSpineIndex,:));
        vNewSpotsSpine.SetColorRGBA((rand(1, 1)) * 256 * 256 * 256 );
        vNewSpotsSpine.SetName([char(vFilaments.GetName), ' spines ' num2str(FilamentIndex)]);
        result1.AddChild(vNewSpotsSpine, -1);
    end
    end
    clearvars vInd vIds vFactors vFactorNames vUnits vNames vDendriteXYZ vSpineXYZ
    clearvars vFilamentsT  vSpineIndex timex Adj yAdj zAdjv %DendriteIndex
    
%Generate the mean Intensity stats for each dendrite

    
    %Adjusting the Filament Points that fall outside the edge of the volume for
    %Spot placement and calculation of Intensities
     xAdj=vFilamentsXYZ(:,1);
     yAdj=vFilamentsXYZ(:,2);
     zAdj=vFilamentsXYZ(:,3);
     xAdj(xAdj<=vImarisApplication.GetDataSet.GetExtendMinX)=vImarisApplication.GetDataSet.GetExtendMinX+0.001;
     xAdj(xAdj>=vImarisApplication.GetDataSet.GetExtendMaxX)=vImarisApplication.GetDataSet.GetExtendMaxX-0.001;
     yAdj(yAdj<=vImarisApplication.GetDataSet.GetExtendMinY)=vImarisApplication.GetDataSet.GetExtendMinY+0.001;
     yAdj(yAdj>=vImarisApplication.GetDataSet.GetExtendMaxY)=vImarisApplication.GetDataSet.GetExtendMaxY-0.001;
     zAdj(zAdj<=vImarisApplication.GetDataSet.GetExtendMinZ)=vImarisApplication.GetDataSet.GetExtendMinZ+0.001;
     zAdj(zAdj>=vImarisApplication.GetDataSet.GetExtendMaxZ)=vImarisApplication.GetDataSet.GetExtendMaxZ-0.001;
     vFilamentsXYZAdj=[xAdj yAdj zAdj];
     vNewSpots.Set(vFilamentsXYZAdj, vTimeIndex, vFilamentsRadius);
     
    %Get Original Stats order vNewSpots
     vAllStatistics = vNewSpots.GetStatistics;
     vFilamentSpots1StatNames = cell(vAllStatistics.mNames);
     vFilamentSpots1StatValues = vAllStatistics.mValues;
     vFilamentSpots1Index1=strmatch('Intensity Center', vFilamentSpots1StatNames);
     vFilamentSpots1Index2=strmatch('Intensity Mean', vFilamentSpots1StatNames);
     vFilamentSpots1IntensityCenter = vFilamentSpots1StatValues(vFilamentSpots1Index1,:);
     vFilamentSpots1IntensityMean = vFilamentSpots1StatValues(vFilamentSpots1Index2,:);
     
     %Get Filament length stats
    if FilamentIndex==0   
        vAllStatisticsFilaments = vFilaments.GetStatistics;
        vFilamentStatNames = cell(vAllStatisticsFilaments.mNames);
        vFilamentStatValues = vAllStatisticsFilaments.mValues;
        vFilamentStatIds = vAllStatisticsFilaments.mIds;
        %Find dendrite lengths all filament from scene object
        vDendriteLengthIndex=strmatch('Dendrite Length', vFilamentStatNames);
        vDendriteLength = vFilamentStatValues(vDendriteLengthIndex,:);
        vDendriteLengthIDs = vFilamentStatIds(vDendriteLengthIndex,:);
    end
    
    %Test and Clear Statistics working variables
    StatsMisMatch='NO';
    if size(vFilamentSpots1IntensityCenter)/vNumberOfChannels~=size(vFilamentsXYZ,1)
        StatsMisMatch='YES';
    end
    clearvars vAllStatistics vFilamentSpots1StatNames vFilamentSpots1StatValues vFilamentSpots1Index1
    clearvars vFilamentSpots1Index2 vAllStatisticsFilaments vFilamentStatNames vFilamentStatValues vFilamentStatIds
    clearvars vDendriteLengthIndex vFilamentsXYZAdj
    
    vDendriteIntensityMeanWorking=[];
    vDendriteIntensityCenterWorking=[];
    vDendriteIntensityCenterFinal=[];
    vDendriteIntensityMeanFinal=[];
    vSpineIntensityCenterFinal=[];
    vSpineIntensityMeanFinal=[]; 
    vSpineIntensityMeanWorking=[];
    vSpineIntensityCenterWorking=[];
    vSpineSegmentIdsWorking=[];
    vDendriteSegmentIdsWorking=[];
    vDendritePositionsFinal=[];
    vSpinePositionsFinal=[];
    vDendriteRadiusFinal=[];
    vSpineRadiusFinal=[];
    vSpineSegmentIds=[];
    vDendriteSegmentIds=[];
    wSegmentIndexFinal=[];
    wSpineSegment=[];
    wDendriteSegment=[];
%%
%
%Loop through dendrite segments, terminal segements and spine segments
%Calculate bouton and place as new Spots object
%isoloate dendrite only
    dcount=1;
    scount=1;
    % Logical array for spots Object detection, setting them all to false
    % per filament object
    vIsClose = false(vSpotsCount, 1);
    for vBranchIndex=1:(vNumberOfDendriteBranches)
        display (vBranchIndex);
        %Set the ID for dendrite segment
        wSegmentIndex = vSegmentIds(vBranchIndex);
        %Logical arguement to identify spots in segment
        vSpotsT = vFilamentsEdgesSegmentId == wSegmentIndex;
        vSpotsIndex = find(vSpotsT');
        %Identify position for dendrite segment
        %Test with new method of filtering        
        vDendriteEdgesWorking=vFilamentsEdges(vSpotsIndex,:);
        vEdgesUnique=unique(vDendriteEdgesWorking);
        vDendritePositionsWorking=vFilamentsXYZ(vEdgesUnique,1:3);
        vDendriteRadiusWorking=vFilamentsRadius(vEdgesUnique,1);
        vTypesWorking=vTypes(vEdgesUnique,:);
%End of the testing        
        
        if vTypesWorking(2)==1
            vSpineSegmentIdsWorking = vFilamentsEdgesSegmentId(vSpotsIndex,:);
            vSpineSegmentIds=[vSpineSegmentIds;vSpineSegmentIdsWorking];
            wSpineSegment=[wSpineSegment;wSegmentIndex];
        else
            vDendriteSegmentIdsWorking = vFilamentsEdgesSegmentId(vSpotsIndex,:);
            vDendriteSegmentIds=[vDendriteSegmentIds;vDendriteSegmentIdsWorking];
            wDendriteSegment=[wDendriteSegment;wSegmentIndex];
            vFilamentPointsXYZFinal=[vFilamentPointsXYZFinal;vDendritePositionsWorking]
            vFilamentPointsRadiusFinal=[vFilamentPointsRadiusFinal;vDendriteRadiusWorking]
        end
        

        if vTypesWorking==1
            %compile Spines spots
            vSpinePositionsFinal=[vSpinePositionsFinal;vDendritePositionsWorking];
            vSpineRadiusFinal=[vSpineRadiusFinal;vDendriteRadiusWorking];
        else
            %compile the reordered spots for dendrites
            vDendritePositionsFinal=[vDendritePositionsFinal;vDendritePositionsWorking];
            vDendriteRadiusFinal=[vDendriteRadiusFinal;vDendriteRadiusWorking];  
        end
        
        %Identify the peaks in dendrite radius compared to surrounding radii        
        %[maxtab, mintab] = peakdet(vDendriteRadiusWorking, 0.3);

        %figure; plot(vDendriteRadiusWorking);
        %plot(mintab(:,1), mintab(:,2), 'g*');
        %hold on; plot(maxtab(:,1), maxtab(:,2), 'r*');
        if vTypesWorking==0 & isequal(qDoBoutonDetection,'YES')
            %Identify the peaks in dendrite radius compared to surrounding radii        
            [maxtab, mintab] = peakdet(vDendriteRadiusWorking, vPeakDetThreshold);
        if  isempty(maxtab)
            vBoutonNumberFinal=[vBoutonNumberFinal;0];
        else
            vBoutonIndex= maxtab(:,1);
            vBoutonPositionXYZ=vDendritePositionsWorking(vBoutonIndex,:);
            vBoutonRadius=vDendriteRadiusWorking(vBoutonIndex,:);
            vBoutonNumber=numel(vBoutonRadius);
            vBoutonTime=zeros(size(vBoutonRadius))+vFilamentsIndexT; 
            vBoutonPositionXYZFinal=[vBoutonPositionXYZFinal;vBoutonPositionXYZ]
            vBoutonRadiusFinal=[vBoutonRadiusFinal;vBoutonRadius];
            vBoutonTimeFinal=[vBoutonTimeFinal;vBoutonTime];
            vBoutonNumberFinal=[vBoutonNumberFinal;vBoutonNumber];
        end
            wSegmentIndexFinal=[wSegmentIndexFinal;wSegmentIndex];
        end
 
%%
%Calculate the mean Intensity values on per dendrite basis        
%generate dendrite intensity values by averageing all spots for each segment, per channel
        if isequal (StatsMisMatch,'NO') & isequal (qIsAFilament,'YES')
        if vTypesWorking(2)==1
            for acount=1:numel(vEdgesUnique)
                n = numel(vFilamentsRadius);%number of point spots in dendrite
                %y = vFilamentSpots1IntensityCenter(1 : n : end);
                y = (vFilamentSpots1IntensityCenter(vEdgesUnique(acount) : n : end))';
                %x = vFilamentSpots1IntensityMean(1 : n : end);
                x = (vFilamentSpots1IntensityMean(vEdgesUnique(acount) : n : end))';            
                vSpineIntensityCenterWorking=[vSpineIntensityCenterWorking;y];
                vSpineIntensityMeanWorking=[vSpineIntensityMeanWorking;x];               
            end
            scount=scount+1;
            vSpineIntensityMeanFinal=[vSpineIntensityMeanFinal;mean(vSpineIntensityMeanWorking)];
            vSpineIntensityCenterFinal=[vSpineIntensityCenterFinal;mean(vSpineIntensityCenterWorking)]; 
        else
            for acount=1:numel(vEdgesUnique)
                n = numel(vFilamentsRadius);%number of point spots in dendrite
                %y = vFilamentSpots1IntensityCenter(1 : n : end);
%Get the nth value, starting at Index defined by vEdgesUnique           
                y = (vFilamentSpots1IntensityCenter(vEdgesUnique(acount) : n : end))';
                %x = vFilamentSpots1IntensityMean(1 : n : end);
                x = (vFilamentSpots1IntensityMean(vEdgesUnique(acount) : n : end))';
                show=(vBranchIndex);
                
                vDendriteIntensityCenterWorking=[vDendriteIntensityCenterWorking;y];
                vDendriteIntensityMeanWorking=[vDendriteIntensityMeanWorking;x];
                display(acount);
            end
            dcount=dcount+1;
            vDendriteIntensityMeanFinal=[vDendriteIntensityMeanFinal;mean(vDendriteIntensityMeanWorking)];
            vDendriteIntensityCenterFinal=[vDendriteIntensityCenterFinal;mean(vDendriteIntensityCenterWorking)];
        end
        end
        vDendriteIntensityMeanWorking=[];
        vDendriteIntensityCenterWorking=[];
        vSpineIntensityMeanWorking=[];
        vSpineIntensityCenterWorking=[];
%%
%Calculate the spot to dendrite segment lengths.
        % Identify Spots at the filament's same time index
        if isequal (qDoSpotsCloseFilaments,'YES') & vTypesWorking==0 & isequal (qIsAFilament,'YES')
        vSpotsIndex1 = find(vSpotsPosT == vFilamentsIndexT);
        vSpotsCountT = numel(vSpotsIndex1); 
        for vIndex1= 1:size(vDendritePositionsWorking,1)
            vDist(:,vIndex1) = sqrt((vSpotsPosXYZ(:,1)-vDendritePositionsWorking(vIndex1,1)).^2 + (vSpotsPosXYZ(:,2)-vDendritePositionsWorking(vIndex1,2)).^2);
            vDist(:,vIndex1) = vDist(:,vIndex1)-vDendriteRadiusWorking(vIndex1);
        end
        vLimit=min(vDist,[],2);% find minimum value for each row for each dendrite position
        vIsClose(vSpotsIndex1) = vLimit < vDistanceLimit;
        vIsCloseFinal=[vIsCloseFinal vIsClose]
        % Calculate the Density of spots per dendrite segment
        vNumberofSpotsDendrite=sum(vIsClose(:));
        CompletevNumberofSpotsDendrite=[CompletevNumberofSpotsDendrite;vNumberofSpotsDendrite];
        clearvars vDist vLimit vIsCloseWorking  
        end
        barcount=numel(vDendriteLength);
        bar1=bar1+1;
        waitbar(bar1/barcount, vProgressDisplayDendrites);
        
        
        
        
    end
    
    %Find dupliates in the dendrite positions delete them from radius and
    %dendrites
    % indices to unique values in column 3
    if isequal (qIsAFilament,'YES');
    [ind2, ind2] = unique(vDendritePositionsFinal(:, 3), 'rows');
    % duplicate indices
    duplicate_ind = setdiff(1:size(vDendritePositionsFinal, 1), ind2);
    % duplicate values
    duplicate_value = vDendritePositionsFinal(duplicate_ind, 3);
    %Delete selected duplicate indices
    vDendriteRadiusFinal(duplicate_ind,:) = [];
    vDendritePositionsFinal(duplicate_ind,:)=[];
    end
    
%%
    %Duplicate Filament duplicate for custom stats
    vFilaments.SetVisible(0);
    if isequal (qIsAFilament,'YES')
    vNewFilament.AddFilament(vFilamentsXYZ, vFilamentsRadius, vTypes, vFilamentsEdgesCopy, vFilamentsIndexT);
    vNewFilament.SetBeginningVertexIndex(FilamentIndex, vFilaments.GetBeginningVertexIndex(FilamentIndex));
    vNewFilament.SetName(sprintf('%s Analysis', char(vFilaments.GetName)));
    vNewFilament.SetColorRGBA(vRGBA);
    end
%%
%Add new statistics
    CompletewDendriteSegment=[CompletewDendriteSegment;wDendriteSegment];
    CompletewSpineSegment=[CompletewSpineSegment;wSpineSegment];

%loop to generate dendrite Intensity statistics for each channel
    if isequal (StatsMisMatch, 'NO') & isequal (qIsAFilament,'YES')
    for stat=1:vNumberOfChannels
        TempMean(:,1)=vDendriteIntensityMeanFinal(:,stat);
        TempCenter(:,1)=vDendriteIntensityCenterFinal(:,stat);
        CompleteCount=[CompleteCount;dcount];
        CompleteTempMean(:,stat)=TempMean;
        CompleteTempCenter(:,stat)=TempCenter;
    end
    CompleteMeanFinal=[CompleteMeanFinal;CompleteTempMean];
    CompleteCenterFinal=[CompleteCenterFinal;CompleteTempCenter];

    clearvars TempMean TempCenter CompleteTempMean CompleteTempCenter
%%    
%loop to generate Spine Intensity statistics for each channel
    if scount > 1
    for stat=1:vNumberOfChannels
        spTempMean(:,1)=vSpineIntensityMeanFinal(:,stat);
        spTempCenter(:,1)=vSpineIntensityCenterFinal(:,stat);
        CompletespCount=[CompletespCount;scount];
        CompletespTempMean(:,stat)=spTempMean;
        CompletespTempCenter(:,stat)=spTempCenter;
    end
    CompletespMeanFinal=[CompletespMeanFinal;CompletespTempMean];
    CompletespCenterFinal=[CompletespCenterFinal;CompletespTempCenter];  
    end
    else
        for stat=1:vNumberOfChannels
            CompletewSpineSegment=[CompletewSpineSegment;wSpineSegment];
            CompletewDendriteSegment=[CompletewDendriteSegment;wDendriteSegment];
        end
    end
    clearvars spTempMean spTempCenter CompletespTempMean CompletespTempCenter
%%
%Add Filament intensity statitsics
        if vCount==FilamentIndex+1
            for stat=1:vNumberOfChannels
                vNewName = sprintf(' Dendrite Intensity Center Mean ch %d', stat);
                vInd = 1:numel(CompletewDendriteSegment);
                vNames(vInd) = {vNewName};
                vFactors(vInd) = {'Dendrite'};
                vFactorNames = {'Category'};
                vUnits(vInd) = { char(vImarisApplication.GetDataSet.GetUnit) };
                vNewFilament.AddStatistics(vNames, CompleteCenterFinal(:,stat), vUnits, vFactors, vFactorNames, CompletewDendriteSegment);
                vNewName = sprintf(' Dendrite Intensity Mean ch %d', stat);
                vNames(vInd) = {vNewName};
                vNewFilament.AddStatistics(vNames, CompleteMeanFinal(:,stat), vUnits, vFactors, vFactorNames, CompletewDendriteSegment);
            end
        end 
        if vCount==FilamentIndex+1 & scount>1
            for stat=1:vNumberOfChannels
                vNewName = sprintf(' Spine Intensity Center Mean ch %d', stat);
                vInd = 1:numel(CompletewSpineSegment);
                vNames(vInd) = {vNewName};
                vFactors(vInd) = {'Spine'};
                vFactorNames = {'Category'};
                vUnits(vInd) = { char(vImarisApplication.GetDataSet.GetUnit) };
                vNewFilament.AddStatistics(vNames, CompletespCenterFinal(:,stat), vUnits, vFactors, vFactorNames, CompletewSpineSegment);
                vNewName = sprintf(' Spine Intensity Mean ch %d', stat);
                vNames(vInd) = {vNewName};
                vNewFilament.AddStatistics(vNames, CompletespMeanFinal(:,stat), vUnits, vFactors, vFactorNames, CompletewSpineSegment);
            end
        end




%%
%Values for dendrite statistics
%TotalSegmentIndex=[TotalSegmentIndex;wSegmentIndexFinal];
if vCount==FilamentIndex+1
    vInd=1:numel(CompletewDendriteSegment);
    vUnits2(vInd) = { char(vImarisApplication.GetDataSet.GetUnit) };
    vFactors2(vInd) = {'Dendrite'};
end
%%
if vCount==FilamentIndex+1
    [xx,xx]=sort(CompletewDendriteSegment);
    [yy,yy] = sort(vDendriteLengthIDs);
    vDendriteLengthFinal(xx)=vDendriteLength(yy);
clearvars xx yy
end
%Add Bouton count per dendrite segment
    if isequal(qDoBoutonDetection,'YES') & isequal (qIsAFilament,'YES')
        vNewName2= sprintf(' Dendrite Bouton Number');
        vNewName3= sprintf(' Dendrite Bouton Density');

        if vCount>0
            %Concatonate all boutons from all filaments
            TotalBoutons=[TotalBoutons;vBoutonNumberFinal];
            %Add Spot for each bouton
            vBoutonPositionAll=[vBoutonPositionAll;vBoutonPositionXYZFinal];
            vBoutonTimeAll=[vBoutonTimeAll;vBoutonTimeFinal];
            vBoutonRadiusAll=[vBoutonRadiusAll;vBoutonRadiusFinal];
            if vCount==FilamentIndex+1
                TotalBoutons=single(TotalBoutons);
                vNames2(vInd) = {vNewName2};
                vNewFilament.AddStatistics(vNames2, TotalBoutons, vUnits2, vFactors2, vFactorNames, CompletewDendriteSegment);
                vNames2(vInd) = {vNewName3};
                vBoutonDensity=TotalBoutons./vDendriteLength;
                vNewFilament.AddStatistics(vNames2, (TotalBoutons)./(vDendriteLengthFinal'), vUnits2, vFactors2, vFactorNames, CompletewDendriteSegment);
                vBoutonTimeAll=vBoutonTimeAll'
                %Add the Bouton Spots
                vNewSpotsBouton.Set(vBoutonPositionAll,vBoutonTimeAll,vBoutonRadiusAll);
                vNewSpotsBouton.SetColorRGBA((rand(1, 1)) * 256 * 256 * 256 );
                vNewSpotsBouton.SetName(['Boutons on  ',char(vFilaments.GetName)])%, num2str(FilamentIndex)]);
                result3.AddChild(vNewSpotsBouton, -1);            
            end
        else
            vNames2(vInd) = {vNewName2};
            vNewFilament.AddStatistics(vNames2, vBoutonNumberFinal, vUnits2, vFactors2, vFactorNames, CompletewDendriteSegment);
            vNames2(vInd) = {vNewName3};
            vNewFilament.AddStatistics(vNames2, (vBoutonNumberFinal)./(vDendriteLengthFinal'), vUnits2, vFactors2, vFactorNames, CompletewDendriteSegment);
            vBoutonTimeFinal=vBoutonTimeFinal';
            vNewSpotsBouton.Set(vBoutonPositionXYZFinal,vBoutonTimeFinal,vBoutonRadiusFinal);
            vNewSpotsBouton.SetColorRGBA((rand(1, 1)) * 256 * 256 * 256 );
            vNewSpotsBouton.SetName(['Boutons on ',char(vFilaments.GetName)])%, num2str(FilamentIndex)]);
            result3.AddChild(vNewSpotsBouton, -1);            
        end
    end
    
%Add Spot count per dendrtie segment
    if isequal(qDoSpotsCloseFilaments,'YES') & isequal (qIsAFilament,'YES')
        vNewName2 = sprintf(' Dendrite Puncta Number');
        vNewName3 = sprintf(' Dendrite Puncta Density');
        if vCount>0
            TotalSpotsperSegment=[TotalSpotsperSegment;CompletevNumberofSpotsDendrite];
            if vCount==FilamentIndex+1
                CompletevNumberofSpotsDendrite=single(CompletevNumberofSpotsDendrite)
                vNames2(vInd) = {vNewName2};
                vNewFilament.AddStatistics(vNames2, CompletevNumberofSpotsDendrite, vUnits2, vFactors2, vFactorNames, CompletewDendriteSegment);
                vNames2(vInd) = {vNewName3};
                vPunctaDensity=(CompletevNumberofSpotsDendrite)./(vDendriteLengthFinal');
                vNewFilament.AddStatistics(vNames2, vPunctaDensity, vUnits2, vFactors2, vFactorNames, CompletewDendriteSegment);          
            end
        else
            vNames2(vInd) = {vNewName2};
            vNewFilament.AddStatistics(vNames2, CompletevNumberofSpotsDendrite, vUnits2, vFactors2, vFactorNames, CompletewDendriteSegment);
            vNames2(vInd) = {vNewName3};
            vPunctaDensity=(CompletevNumberofSpotsDendrite)./(vDendriteLengthFinal');
            vNewFilament.AddStatistics(vNames2, vPunctaDensity, vUnits2, vFactors2, vFactorNames, CompletewDendriteSegment);          
 
        end    
    end

if isequal(qDoSpotsCloseFilaments,'YES') & vCount==FilamentIndex+1 & isequal (qIsAFilament,'YES')
% hide original spots
vSpots.SetVisible(false);

% add new spots (close)
vSpotsA = vImarisApplication.GetFactory.CreateSpots;
vIsCloseFinal=any(vIsCloseFinal,2)
% The radius can be setted to zero because it is overwritten on the next line
% Only an array with the right size is needed
vSpotsA.Set(vSpotsPosXYZ(vIsCloseFinal,:), vSpotsPosT(vIsCloseFinal), zeros(sum(vIsCloseFinal),1));
vSpotsA.SetRadiiXYZ(vSpotsRadius(vIsCloseFinal,:));
vSpotsA.SetName(sprintf('%s closer than %0.1f um to %s edge', char(vSpots.GetName), vDistanceLimit, char(vFilaments.GetName)));
vSpotsA.SetColorRGBA(255);
%vImarisApplication.GetSurpassScene.AddChild(vSpotsA, -1);
result3.AddChild(vSpotsA, -1);            

% add new spots (far)
vSpotsB = vImarisApplication.GetFactory.CreateSpots;
vIsFar = ~vIsCloseFinal;
vSpotsB.Set(vSpotsPosXYZ(vIsFar,:), vSpotsPosT(vIsFar),  zeros(sum(vIsFar),1));
vSpotsB.SetRadiiXYZ(vSpotsRadius(vIsFar,:));
vSpotsB.SetName(sprintf('%s farther than %0.1f um to %s edge', char(vSpots.GetName), vDistanceLimit, char(vFilaments.GetName)));
vSpotsB.SetColorRGBA(255*256*256);
%vImarisApplication.GetSurpassScene.AddChild(vSpotsB, -1);   
result3.AddChild(vSpotsB, -1);            
end

    
    clearvars vNames vUnits vFactors
%%
%Add new spots and filament to the scene
%vNewSpots.Set(vFilamentPointsXYZFinal,time(vDendriteIndex,:)',vFilamentPointsRadiusFinal);
%result1.AddChild(vNewSpots, -1);



    result3.AddChild(vNewFilament, -1);
    vImarisApplication.GetSurpassScene.AddChild(result3, -1);
    result1.SetVisible(0);
    vImarisApplication.GetSurpassScene.AddChild(result1, -1);
    clearvars vTimeIndex
end
close(vProgressDisplayDendrites);

%%
if isequal(qFindFilamentsPoints,'YES')
% generate spots
vParent = vImarisApplication.GetFactory.CreateDataContainer;
vParent.SetName([char(vFilaments.GetName), '_Points ' num2str(FilamentIndex)]);
vTracker = vImarisApplication.GetImageProcessing;
for vField = 1:vFieldsCount
  vFieldName = vFieldNames{vField};
    
  vSizes = 0:vSizeT;
  for vTime = 1:vSizeT
    vSizes(vTime + 1) = vSizes(vTime) + size(vData(vTime).(vFieldName), 1);
  end
  vXYZRT = zeros(vSizes(vSizeT), 5);
  for vTime = 1:vSizeT
    vTimes = vSizes(vTime) + 1: vSizes(vTime + 1);
    vXYZRT(vTimes, 1:4) = vData(vTime).(vFieldName);
    vXYZRT(vTimes, 5) = vTime - 1;
  end
  
  vSpots = vImarisApplication.GetFactory.CreateSpots;
  vSpots.Set(vXYZRT(:, 1:3), vXYZRT(:, 5), vXYZRT(:, 4));

  vSpots.SetName([char(vFilaments.GetName), '_', vFieldName]);
  vColor = hsv2rgb([vField/vFieldsCount, 1, 1]);
  vSpots.SetColorRGBA(round(vColor * 255) * [1; 256; 256*256]);
  
  % update scene
  vParent.AddChild(vSpots, -1);

end

%%
%vImarisApplication.GetSurpassScene.AddChild(result, -1);
%result.SetVisible(0);
%%
%Add point track data    
vParent.SetVisible(0);
vImarisApplication.GetSurpassScene.AddChild(vParent, -1);
end
else
delete(f);
end
%vFilaments.SetVisible(0);



function aData = CollectFilamentsData(aFilamentsXYZ, aFilamentsRadius, ...
  aFilamentsEdges, aFilamentsSpines, aRoot)

[vIsDendriteBranch, vIsDendriteEndPoint] = GetPoints(aFilamentsEdges, ~aFilamentsSpines);
[vIsSpineBranch, vIsSpineEndPoint] = GetPoints(aFilamentsEdges, aFilamentsSpines);

if aRoot > 0
  vIsDendriteBranch(aRoot) = false;
  vIsDendriteEndPoint(aRoot) = false;
else 
  aRoot = [];
end

vIsSpineRoot = GetMixedPoints(aFilamentsEdges, ~aFilamentsSpines);
vIsDendriteEndPoint(vIsSpineRoot) = false;
for vIndex = find(vIsSpineEndPoint)
  vCount = numel(find(aFilamentsEdges == vIndex, 2));
  if vCount > 1
    vIsSpineEndPoint(vIndex) = false;
  end
end

aData = struct( ...
  'BeginningPoint', ...
  [aFilamentsXYZ(aRoot, :), aFilamentsRadius(aRoot)], ...
  'DendriteBranch', ...
  [aFilamentsXYZ(vIsDendriteBranch, :), aFilamentsRadius(vIsDendriteBranch)], ...
  'DendriteTerminal', ...
  [aFilamentsXYZ(vIsDendriteEndPoint, :), aFilamentsRadius(vIsDendriteEndPoint)], ...
  'AttachmentPoint', ...
  [aFilamentsXYZ(vIsSpineRoot, :), aFilamentsRadius(vIsSpineRoot)], ...
  'SpineBranch', ...
  [aFilamentsXYZ(vIsSpineBranch, :), aFilamentsRadius(vIsSpineBranch)], ...
  'SpineTerminal', ...
  [aFilamentsXYZ(vIsSpineEndPoint, :), aFilamentsRadius(vIsSpineEndPoint)]);

function [aIsBranch, aIsEndPoint] = GetPoints(aEdges, aIsType)

vPointsSize = numel(aIsType);
vTypeIndices = find(aIsType);
vSize = numel(vTypeIndices);

vIsEdgeLeft = aIsType(aEdges(:, 1));
vIsEdgeRight = aIsType(aEdges(:, 2));
vIsEdge = vIsEdgeLeft & vIsEdgeRight;
vEdges = aEdges(vIsEdge, :);

aIsBranch = false(1, vPointsSize);
aIsEndPoint = false(1, vPointsSize);
for vIndex = 1:vSize
  vTypeIndex = vTypeIndices(vIndex);
  vCount = numel(find(vEdges == vTypeIndex, 3));
  if vCount == 1
    aIsEndPoint(vTypeIndex) = true;
  elseif vCount > 2
    aIsBranch(vTypeIndex) = true;
  end
end

function aMixedPoints = GetMixedPoints(aEdges, aIsType)

vIsEdgeLeft = aIsType(aEdges(:, 1));
vIsEdgeRight = aIsType(aEdges(:, 2));
vEdgesLeft = vIsEdgeLeft & ~vIsEdgeRight;
vEdgesRight = ~vIsEdgeLeft & vIsEdgeRight;
aMixedPoints = [aEdges(vEdgesLeft, 1)', aEdges(vEdgesRight, 2)'];

function [maxtab, mintab]=peakdet(v, delta, x)
%PEAKDET Detect peaks in a vector
%        [MAXTAB, MINTAB] = PEAKDET(V, DELTA) finds the local
%        maxima and minima ("peaks") in the vector V.
%        MAXTAB and MINTAB consists of two columns. Column 1
%        contains indices in V, and column 2 the found values.
%      
%        With [MAXTAB, MINTAB] = PEAKDET(V, DELTA, X) the indices
%        in MAXTAB and MINTAB are replaced with the corresponding
%        X-values.
%
%        A point is considered a maximum peak if it has the maximal
%        value, and was preceded (to the left) by a value lower by
%        DELTA.

% Eli Billauer, 3.4.05 (Explicitly not copyrighted).
% This function is released to the public domain; Any use is allowed.

maxtab = [];
mintab = [];

v = v(:); % Just in case this wasn't a proper vector

if nargin < 3
  x = (1:length(v))';
else 
  x = x(:);
  if length(v)~= length(x)
    error('Input vectors v and x must have same length');
  end
end
  
if (length(delta(:)))>1
  error('Input argument DELTA must be a scalar');
end

if delta <= 0
  error('Input argument DELTA must be positive');
end

mn = Inf; mx = -Inf;
mnpos = NaN; mxpos = NaN;

lookformax = 1;

for i=1:length(v)
  this = v(i);
  if this > mx, mx = this; mxpos = x(i); end
  if this < mn, mn = this; mnpos = x(i); end
  
  if lookformax
    if this < mx-delta
      maxtab = [maxtab ; mxpos mx];
      mn = this; mnpos = x(i);
      lookformax = 0;
    end  
  else
    if this > mn+delta
      mintab = [mintab ; mnpos mn];
      mx = this; mxpos = x(i);
      lookformax = 1;
    end
  end
end
%%


%end



